@@ -89,14 +89,36 @@ class LazyStrategy(SearchStrategy):
     def calc_is_empty(self):
         return self.wrapped_strategy.is_empty
 
+    def calc_has_reusable_values(self):
+        return self.wrapped_strategy.has_reusable_values
+
     @property
     def wrapped_strategy(self):
         if self.__wrapped_strategy is None:
             with self.__settings:
-                self.__wrapped_strategy = self.__function(
-                    *[unwrap_strategies(s) for s in self.__args],
-                    **{k: unwrap_strategies(v)
-                       for k, v in self.__kwargs.items()})
+                unwrapped_args = tuple(
+                    unwrap_strategies(s) for s in self.__args)
+                unwrapped_kwargs = {
+                    k: unwrap_strategies(v)
+                    for k, v in self.__kwargs.items()
+                }
+
+                base = self.__function(
+                    *self.__args, **self.__kwargs
+                )
+                if (
+                    unwrapped_args == self.__args and
+                    unwrapped_kwargs == self.__kwargs
+                ):
+                    self.__wrapped_strategy = base
+                else:
+                    self.__wrapped_strategy = self.__function(
+                        *unwrapped_args,
+                        **unwrapped_kwargs)
+                    self.__wrapped_strategy.force_has_reusable_values = \
+                        base.has_reusable_values
+                    assert self.__wrapped_strategy.has_reusable_values == \
+                        base.has_reusable_values
         return self.__wrapped_strategy
 
     def do_validate(self):
